#include "TreeEvaporationCalc.h"

/*************************************************************************** 
Function Name:  treeEvaporation function 
Purpose: To calculate:
- The evaporation of water from the leaves
Updates:
- The evaporation of water and snow from the leaves
Requires:		int timeStep - the timestep value
DataFolder *folder - the key number for the folder being processed
Returns:		Void

Reference:   1. Wang, Jun, Theodore A. Endreny, and David J. Nowak, 2008.
Mechanistic Simulation of Tree Effects in an Impervious Water Balance Modell 
Journal of the American Water Resources Association (JAWRA) 44(1): 75-85
DOI: 10.1111/j.1752-1688.2007.00139.x
2. Yang, Yang, Theodore A. Endreny, and David J. Nowak, 2011. 
iTree-Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model. 
Journal of the American Water Resources Association (JAWRA) 47(6):12111218.
DOI: 10.1111/j.1752-1688.2011.00564.x

Comments:	

Author(s):		Jun Wang, jwang06@mailbox.syr.edu, V4/V4.5
Yang Yang, yyang31@syr.edu, V5
 	
Thomas Taggart, traggatmot@gmail.com

*****************************************************************************/

void TreeEvaporationCalc::calculateTreeEvaporationTemperature(Inputs *input, DataFolder *folder, int timeStep)
{
    #define lamda 2257000.0
	folder->VarDict["treeEvap"] = folder->VarDict["TreeLEE"] / lamda;
	if (folder->VarDict["treeEvap"] > folder->VarDict["treeStoredW"])
		folder->VarDict["treeEvap"] = folder->VarDict["treeStoredW"];
	folder->VarDict["treeStoredW"] = folder->VarDict["treeStoredW"] - folder->VarDict["treeEvap"];
	folder->VarDict["totalTreeEvap_m"] += folder->VarDict["treeEvap"];
}

void TreeEvaporationCalc::calculateTreeEvaporationStatistical(Inputs *input, DataFolder *folder, int timeStep)
{
	double TreeEvapSnow = 0.0;        // Initialize TreeEPSnow (amount of evaporated snow) = 0 
	double TreeEvapW = 0.0;           // Initialize TreeEPW (amount of evaporated water) = 0
	double TreeEvap = 0.0;                // Initialize TreeEP (potential evaporation) = 0

	// TreeEPSnow (the amount of evaporated snow) = PETreeSnow (the potential evaporation of snow) 
	TreeEvapSnow = input->PETreeSnow[timeStep];

	//routine to evaporate liquid water after evaporating any snow water equivalent
	//enter conditional if potential evaporation for snow water exceeds stored snow water
	if (TreeEvapSnow > folder->VarDict["treeStoredSWE"]) {
		//calcluate fraction of potential evaporation utilzed by snow water evaporation
		double f = folder->VarDict["treeStoredSWE"] / TreeEvapSnow;
		TreeEvapSnow = folder->VarDict["treeStoredSWE"];
		//evaporate fraction of water stored in tree
		//convert storMaxTree to value greater than 0 to avoid division by zero
		if (folder->VarDict["storMaxTree"] <= 0) folder->VarDict["storMaxTree"] = 1.0e-10;
		//evaporate all of the snow water, which may be 0 when treeStoredSWE is 0
		TreeEvapW = (1.0 - f) * input->PETree[timeStep] * pow(folder->VarDict["treeStoredW"] / folder->VarDict["storMaxTree"], (2.0 / 3.0));
		if (TreeEvapW > folder->VarDict["treeStoredW"])
			TreeEvapW = folder->VarDict["treeStoredW"];
		//reduce the tree stored water by the tree evaporation of water
		folder->VarDict["treeStoredW"] = folder->VarDict["treeStoredW"] - TreeEvapW;
	}
	//reduce the tree stored snow by the tree evaporation of snow
	//if treeStoredSWE was 0, then TreeEvapSnow was set to 0 in the above condition
	folder->VarDict["treeStoredSWE"] = folder->VarDict["treeStoredSWE"] - TreeEvapSnow;

	TreeEvap = TreeEvapSnow + TreeEvapW;
	
	folder->VarDict["treeEvap"] = TreeEvap;
	folder->VarDict["treeEvapW"] = TreeEvapW;
	folder->VarDict["treeEvapSWE"] = TreeEvapSnow;

	// Updating totals
	folder->VarDict["totalTreeEvapW"] += folder->VarDict["treeEvapW"];
	folder->VarDict["totalTreeEvapSWE"] += folder->VarDict["treeEvapSWE"];
	folder->VarDict["totalTreeEvap_m"] += folder->VarDict["treeEvap"];
	folder->VarVecDict["treeStoredW"].emplace_back(folder->VarDict["treeStoredW"]);
	folder->VarVecDict["treeStoredSWE"].emplace_back(folder->VarDict["treeStoredSWE"]);

	folder->VarVecDict["treeEvapW"].emplace_back(folder->VarDict["treeEvapW"]);
	folder->VarVecDict["treeEvapSWE"].emplace_back(folder->VarDict["treeEvapSWE"]);
	folder->VarVecDict["treeEvap"].emplace_back(folder->VarDict["treeEvap"]);

}



