#include "InfiltrationPowDecayCalc.h"
#include <iostream>

// SoilInfiltration functions 
// History: Introduced by June Wang for use in iTree V4, adapted from Beven (1984) and Beven et al.(1995)
//          'In considering partition between ponding, infiltration, and runoff UFORE-Hydro uses the modified Green-Ampt infiltration theory together with
//          TOPMODEL concepts of infiltration and saturation excess processes (Beven, 1984; Beven et al., 1995).'
// Reference: MECHANISTIC SIMULATION OF TREE EFFECTS IN AN Impervious WATER BALANCE MODEL
// Authors: Jun Wang, Ted Endreny, Dave Nowak
// Reference: A flexible modeling package for topographically based watershed hydrology
// Authors: Jun Wang, James Hasset, Ted Endreny

void SoilInfiltrationPowDecayCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	double R;
	//if (folder->ParamStringDict["Type"]  == "RainGarden")
	//{
	//	R = folder->VarDict["waterOnRainGardenArea"];
	//}
	//else if (folder->ParamStringDict["Type"]  == "GreenRoof")
	//{
	//	R = folder->VarDict["waterOnGreenRoofArea"];
	//}
	//else
	//{
		//Arpit
		//double pMacro = input->paramValues->getParameter("pMacro");				// grabbing the pMacro fraction
		double pMacro = folder->ParamDict["PMacro_frac"];				// grabbing the pMacro fraction
		R = folder->VarDict["runoffToInfil"] * (1.0 - pMacro);		// Precip_mpdt/Runoff on pervious to infiltrate - what cannot be stored in pervious area depression (Runoff from impervious area + water on pervious area) 

		if (folder->ParamStringDict["Type"] != "BulkArea") {
			// From the GI_Outlets.cpp
			R = folder->VarDict["surfaceDepth"] * (1.0 - pMacro);
		}

		folder->VarDict["pMacroFlow"] = folder->VarDict["runoffToInfil"] * pMacro;
		folder->VarDict["runoffToInfil_postPMacro"] = R;
		if (input->SimulationStringParams["ExtendedOutputsTimeStep"] != "NoExtendedOutputs")
		{
			folder->VarVecDict["runoffToInfil_postPMacro"].emplace_back(R);
		}
	//}
	
	

	//Arpit
	//double DT  = input->SimulationNumericalParams["DT_SimulationTimeInterval"];		// Timestep (in hours?) i.e. 1 = 1 hour?
	double DT = 1.0;//input->SimulationNumericalParams["DT_SimulationTimeInterval"];		// Timestep (in hours?) i.e. 1 = 1 hour?
	double PP = R/DT;												// Runoff to infiltrate / timestep - Timestep is usually 1 hour
	
	
    if(timeStep > 0)    // If it is not the first timestep
	{
		// temp = total infiltrated so far (CumInfilQ) - 0.1 * (BaseFlow + SumRET), SumRET = RootZone Evapotransipration
		// CumInfilQ is reduced by 1/10th of (Base flow + SumRET)
		// Q: Why 1/10th the sum of Baseflow and Rootzone ET?  Why not reduce it by the total amount lost to baseflow or Rootzone ET?
		// Is it 1/10th reduction or is it converting from cm to mm?
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Due to the questions above, the following changes were made by Yang
		// She set cumulative infiltration = intial rootzone deficit  - current (previous ts) root zone deficit
		// Two Q's:	1) Why is it hard coded with a 0?  
		//					It is this way beacuse the rootzone storage deficits should be the same for TI bin in a semi-distirbuted run
		//					And in a semi-distrbuted run, the RzsDeficit vector stores only one value
		//			2) Why initial minus previous TS?
		//					No good reason for that right now - guess it shows the cumulative infiltation of the entire run upto the current timestep,
		//					but why from the 1st hour to the current hour?  Should't the window for calculating the cumulative infitlration be a little more specific?
		//double temp = input->paramValues->getParameter("SD0") - repo->rootZone.getRzsDeficit(folder, 0);
		// End of changes by Yang
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		//folder->VarDict[cumInfil = folder->VarDict[cumInfil - 0.1 * (folder->VarDict[baseFlow + folder->VarDict[sumET);
		
		double tempLoss = (folder->VarDict["prevRootZoneQ"] + folder->VarDict["prevSumET"]);
		
		folder->VarDict["cumInfil"] = folder->VarDict["cumInfil"] - tempLoss;

		// Set CumInfilQ to 0, if it is less than 0
		if(folder->VarDict["cumInfil"] < 0.0)
			folder->VarDict["cumInfil"] = 0.0;
	}

	// If the (surface) hydraulic conductivity = 0, infiltration cannot occur (we have an impervious area?) 
	// and the runoff/precip should be passed to the InfilexQ variable for use in the infiltration excess routine
	if(folder->ParamDict["K0_mph"] == 0.0)
	{
		    // All runoff is passed to InfilexQ - the infiltration excess runoff  
		    // The portion of the watershed that is cotrolled by infiltration excess runoff is determined by FracInfiltA in Param.dat
			folder->VarDict["infilExRunoff"] = R;
			// Nothing is infiltrated at this timeStep (and ever?)
			folder->VarDict["cumInfil"] = 0.0;
	}
	// If there is no runoff to infiltrate, run the infiltration routine that handles a "no ponding" situation
	else if(R == 0.0)  
	{
		// Run the noPonding function with the runoff = 0
		noPonding(R, input, folder, timeStep);
	}
	// If Ponding is > 0, run the infiltration routine that handles infiltration when ponding is occurring
	else if(folder->VarDict["ponding"] > 0)  
	{
		// Run the inPonding function with PP (the Runof/timestep increment (DT)) and the timestep increment (DT)
		inPonding(timeStep, PP, DT, folder, input);
	}
	// If CumInfilQ equals 0
	else if(folder->VarDict["cumInfil"] == 0.0)   
	{
		// Run the estimateTP function with runoff (R) and the timestep increment (DT)
		// This function estimates the ponding time, Tp
		estimateTP(timeStep, R, DT, folder, input);
	}
	else  // else run the beforePonding function with Precip_mpdt/Runoff (R) and the timestep increment (DT)
		beforePonding(timeStep, R, DT, folder, input);

	// Suggested Additon by Yang to address problems with the infiltration code that caused a jump in the infiltration excess overflow
	// Still needs to be checked out to see if it has the desired effect
	folder->VarDict["ponding"] = false;



	infilEXQ(R, input, folder, timeStep);	
	
	folder->VarDict["infilExInfil"] = folder->VarDict["infilQ"];
	folder->VarDict["adjInfilExRunoff"] = folder->VarDict["infilExRunoff"] * folder->ParamDict["InfiltExcessGovernedArea_frac"];

	//Reza: Here we calculate the effective infiltration from the contributing area as a function of the "infilExRunoff"
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		input->RepoDict["infilExRunoff_temp"] = folder->VarDict["infilExRunoff"];
	}
	if (folder->ParamStringDict["Type"] != "BulkArea") {

		input->RepoDict["infilExRunoffContArea"] = input->RepoDict["infilExRunoff_temp"] *
			((folder->ParamDict["upSlopeArea"] * //(1.0 - folder->ParamDict["upSlopeAreaImpFrac"]) *
				folder->ParamDict["PerAreaTreated_frac"]));// /
		//		(input->SimulationNumericalParams["CatchmentArea_m2"] - (folder->ParamDict["Area"] * folder->ParamDict["giNumber"])));
		// convert to GI area 
		input->RepoDict["infilExRunoffContArea_GI"] = (input->RepoDict["infilExRunoffContArea"]) /
			(folder->ParamDict["Area"] * folder->ParamDict["giNumber"]);
		
	
		// adding the infilExRunoff of the GI to the infilExInfil of the next timestep
		if (timeStep != 0) {
			folder->VarDict["waterToPerDep_pre"] += input->RepoDict["infilExRunoff_tsi"];
		}
		input->RepoDict["infilExRunoff_tsi"] = folder->VarDict["infilExRunoff"];
		folder->VarDict["infilExRunoff"] = 0.0;
	}
	//if (folder->ParamStringDict["Type"] == "BulkArea") {
	//	folder->VarDict["infilExRunoff"] -= input->RepoDict["infilExRunoffContArea_GI"];
	//	if (folder->VarDict["infilExRunoff"] < 0.0)
	//		folder->VarDict["infilExRunoff"] = 0.0;
	//}

	//Reza
	//if (folder->ParamStringDict["Type"] != "BulkArea") {
	//	folder->VarDict["adjGI_infilExRunoff"] = folder->VarDict["infilExRunoff"];
	//	folder->VarDict["adjInfilExRunoff"] = 0;
	//}
	
	//Storing the infiltrated water at this timestep for this folder
	//Arpit
	if (input->SimulationStringParams["Model"]=="StatisticalHydro"){
		folder->VarVecDict["infilExInfil"].emplace_back(folder->VarDict["infilQ"]);
		folder->VarVecDict["adjInfilExInfil"].emplace_back(folder->VarDict["infilQ"] * folder->ParamDict["InfiltExcessGovernedArea_frac"]);
		folder->VarVecDict["infilExRunoff"].emplace_back(folder->VarDict["infilExRunoff"]);
		folder->VarVecDict["adjInfilExRunoff"].emplace_back(folder->VarDict["adjInfilExRunoff"]);
	}


}



// If there is no Runoff (R = 0), the noPonding function is run
void SoilInfiltrationPowDecayCalc::noPonding(double npR, Inputs *input, DataFolder *folder, int timeStep)
{
	// modify Ponding (in the folder) to be 0
	folder->VarDict["ponding"] = 0;
	// modify InfilQto (in the folder) be npR (=0 (R) in this case) 
	folder->VarDict["infilQ"] = npR;
	
	// if the InfilQ < 0, set InfilQ equal to 0
	if(folder->VarDict["infilQ"] < 0.0)
		folder->VarDict["infilQ"] = 0.0;
	
	// Update the cumulative InfilQ (in the folder) to be the previously calculated CumInfilQ + the just set InfilQ
    double temp =  folder->VarDict["cumInfil"] + folder->VarDict["infilQ"];
	folder->VarDict["cumInfil"] = temp;
}



//  Calculate infiltration excess Pervious flow
void SoilInfiltrationPowDecayCalc::infilEXQ(double ifR, Inputs *input, DataFolder *folder, int timeStep)
{
	// Set Infiltration excess Pervious flow = 0
	folder->VarDict["infilExRunoff"] = 0.0;
	// Set Infiltration excess Pervious flow (in the folder) = ( Runoff (ifR) - amount of infiltrated flow (in the folder, at this timestep))
	folder->VarDict["infilExRunoff"] = ifR - folder->VarDict["infilQ"];

	// If the infiltration excess Pervious flow < 0, set it equal to 0
	if(folder->VarDict["infilExRunoff"] < 0.0)
		folder->VarDict["infilExRunoff"] = 0.0;

	
}



//------------------- infiltration process control ----------------//
void SoilInfiltrationPowDecayCalc::beforePonding(int bIT,  double bR, double bDT, DataFolder *folder, Inputs *input)
{
	
	double bPP = bR/bDT;      // throughflow rate (precipitation after interception)
	// Tom 9/9/15: Can leave these in the infiltration repo for now, but should consider
	// moving them to the folder when we go organizerded with soil data sets
	double mM = folder->ParamDict["m"];    // Scale parameter of soil transmissivity (M)
	double K0 = folder->ParamDict["K0_mph"];         // Surface hydraulic conductivity (K0)
	double fF = 1/mM;   // Hydraulic conductivity control factor = (1/mM) = (1/M)
	double nN = folder->ParamDict["n"];    // Scale parameter of power function (N)

	double F1 = 0.0;
	F1 = folder->VarDict["cumInfil"];  // cumulative infiltration
	double RR = 0.0;
	if(F1 >= mM)    // mM = 1/fF
	{
		F1 = 0.999 * mM;           
		RR = 0.001 * mM;              // can not nearly infiltrated any water
		folder->VarDict["cumInfil"] = 0.999 * mM;
		folder->VarDict["infilQ"] = 0.001 * mM;
		// Edited by Tom - 8-16-16: check to make sure the infiltrated water (at this point it is set equal to 0.001*mM, but that might be greater than BP
		if (folder->VarDict["infilQ"] > bPP)
			folder->VarDict["infilQ"] = bPP;

	}
	else
	{
		if( nN <=1.0 )
			RR = -K0*fF*((folder->ParamDict["WFS_m"]*folder->ParamDict["DWSMI_m"])+F1)/log(1-F1*fF); //WFSuction*DWSMI
		else
			RR = K0*((folder->ParamDict["WFS_m"]*folder->ParamDict["DWSMI_m"])+F1)*(1-F1*fF)/F1; 

		if(RR < bPP)
		{
			folder->VarDict["pondTime"] = (bIT - 1) * bDT;            
			
			if(folder->VarDict["pondTime"] < 0.0)
				folder->VarDict["pondTime"] = 0.0;
			
			folder->VarDict["ponding"] = 1;
			folder->VarDict["IP"] = folder->VarDict["cumInfil"];

			inPonding(bIT,bPP,bDT, folder, input);
		}
		else 
		{
			estimateTP(bIT,bR,bDT, folder, input);     // estimate ponding start time 
		}
	}


}


//------------------- estimate ponding start time ----------//
void SoilInfiltrationPowDecayCalc::estimateTP(int tpIT,  double tpR, double tpDT, DataFolder *folder, Inputs *input)
{
	
	double tpPP = tpR/tpDT;
	
	double mM = folder->ParamDict["m"];   // Scale parameter of soil transmissivity (M)
	double K0 = folder->ParamDict["K0_mph"];  // Surface hydraulic conductivity
	double fF = 1/mM;  // Hydraulic conductivity control factor = (1/mM) = (1/M)
	double nN = folder->ParamDict["n"];   // Scale parameter of power function (N)
	double CSSF = (folder->ParamDict["WFS_m"]*folder->ParamDict["DWSMI_m"]);    // CSSF = WFSuction*DWSMI

	folder->VarDict["pondTime"] = 0.0;
	double F1 = 0.0;
	double RR = 0.0;
	double E = 0.00001;
	double FF = 0.0;
	double F2 = 0.0;
	F2 = folder->VarDict["cumInfil"] + tpPP;
	
	if( F2 == 0.0)
		noPonding(tpR, input, folder, tpIT);
	else 
	{
		if(F2 >= mM)
		{
			//RR = 0.01*mM;						// nearly no infiltration - Tom T: feel like this should be 0.001 to match the 0.999
			RR = 0.001*mM;						// Added by Tom T 9-9-16, see above
			F2 = 0.999*mM;
		}
		else
		{
			if(nN <=1.0)
				RR = -K0*fF*(CSSF+F2)/log(1-F2*fF); 
			else
				RR = K0*(CSSF+F2)*(1-F2*fF)/F2; 
		}


		if(RR > tpPP)
			noPonding(tpR, input, folder, tpIT);
		else
		{
			folder->VarDict["F"] = folder->VarDict["cumInfil"] + tpDT * RR;
			if(folder->VarDict["F"] >= 1/fF)
				folder->VarDict["F"] = 0.999 * 1/fF; // F <= m
			for(int i=0; i<20; i++)
			{
				if(nN <=1.0)
					RR = -K0*fF*(CSSF+F2)/log(1-F2*fF); 
				else
					RR = K0*(CSSF+F2)*(1-F2*fF)/F2;  

				if(RR > tpPP)
				{
					F1 = folder->VarDict["F"];
					folder->VarDict["F"] = (0.5*(F2 + folder->VarDict["F"]));
					FF = F1;
				}
				else
				{
					F2 = folder->VarDict["F"];
					folder->VarDict["F"] = (0.5*(F1 + folder->VarDict["F"]));
					FF = F2;
				}
				if(fabs(folder->VarDict["F"] - FF) < E)
				{
					folder->VarDict["pondTime"] = ((tpIT - 1) * tpDT + (folder->VarDict["F"] - folder->VarDict["cumInfil"])/tpPP);
					
					if(folder->VarDict["pondTime"] > tpIT*tpDT )
					{
						noPonding(tpR, input, folder, tpIT);
						break;
					}
					
					folder->VarDict["IP"] = folder->VarDict["F"];
					folder->VarDict["ponding"] = 1;				// Setting ponding to true
					
					inPonding(tpIT,tpPP,tpDT,folder, input);
					break;
				}
			
			}
		
		}
	
	}
}


// This function infiltrates any ponded water
// It is passed (1) ipTT = timestep, (2) iPP = the Runof/timestep increment (DT), (3) ipDT = timestep increment (DT), and (4) folder = the folder
void SoilInfiltrationPowDecayCalc::inPonding(int ipIT, double ipPP, double ipDT, DataFolder *folder, Inputs * input)
{
	double CC= 0.0;
	double mM = folder->ParamDict["m"];   // Scale parameter of soil transmissivity (M)
	double K0 = folder->ParamDict["K0_mph"];        // Surface hydraulic conductivity
	double fF = 1/mM;  // Hydraulic conductivity control factor = (1/mM) = (1/M)
	double nN = folder->ParamDict["n"];   // Scale parameter of power function (N)
	double CSSF = (folder->ParamDict["WFS_m"]*folder->ParamDict["DWSMI_m"]);    // CSSF = WFSuction*DWSMI
	double tt = ipIT*ipDT - folder->VarDict["pondTime"];  // tt = (timestep * timestep increment) - TP   (Tp? Total Ponded?)

	// If the scale parameter of the power function is <= 1
	if(nN <= 1.0)
		CC= K0/mM * tt;   // CC = hydraulic conductivity / scale parameter of the soild transmissivity * tt
	else
		CC= mM/(K0 * (mM + CSSF));    // CC = scale parameter of the soild transmissivity/ (hydraulic conductivity *(scale parameter of the soil transmissivity + CSSF (WFSuction*DWSMI)))

	double EE = 10.0;     // EE = 10    EE?
	double IR = 0.0;      // IR = 0.0   IR?   Infiltration rate?

	double F1 = folder->VarDict["IP"];  // infiltration at ponding time - but should it be cumulative infiltration at ponding time?
	double F2 = 0.0;
	F2 = folder->VarDict["cumInfil"] + ipDT * ipPP;
	
	if(F2 >= mM)
		F2 = 0.999*mM;                    // cumlative I <= m
	
	folder->VarDict["F"] = F2;

	if(folder->VarDict["IP"] >= mM)
		folder->VarDict["IP"] = 0.999 * mM;   

	if(F2 < folder->VarDict["IP"])
		//cout<<"Non-Critical Warning: Time until ponding and infiltration during ponding numerical solution error at time step "<<ipIT<<"."<<endl;

	folder->VarDict["infilQ"] = 0.0;
		
	for(int i=0; i<45; i++)
	{

		if(folder->VarDict["F"] >= mM)
			folder->VarDict["F"] = 0.999 * mM; 
		
		if(nN <= 1.0)
		{
			simpsonIntegration(folder->VarDict["IP"], folder->VarDict["F"], IR, folder, input);   // Simpson integration
			EE = IR - CC;
		}
		else
			EE = CC*( -CSSF*log((CSSF + folder->VarDict["F"])/(CSSF + folder->VarDict["IP"])) + mM * log((1 - folder->VarDict["IP"] * fF)/(1 - folder->VarDict["F"] * fF)) ) - tt;


		if(folder->VarDict["F"] == 0.999 * mM && EE < 0.0)
		{
			folder->VarDict["F"] = 0.999*mM;
			//cout<<"Non-Critical Warning: Infiltration rate is maxed out at " << folder->VarDict[F <<" m at time step "<<ipIT<<"."<<endl;	 
			break;
		}

		if(EE > 0.0)
		{
	       F2 = folder->VarDict["F"];
		   folder->VarDict["F"] = 0.5*(F1 + folder->VarDict["F"]);
		   if(EE < 0.00001)
			   break;
		}
		else
		{
		   F1 = folder->VarDict["F"];
		   folder->VarDict["F"] = 0.5* (F2 + folder->VarDict["F"]);
		   if(fabs(EE) < 0.00001)
			   break;
		}

	}
	
	folder->VarDict["infilQ"] = folder->VarDict["F"] - folder->VarDict["cumInfil"];	
	folder->VarDict["cumInfil"] = folder->VarDict["F"];
	
	if(folder->VarDict["cumInfil"] >= mM)
		folder->VarDict["cumInfil"] = 0.999 * mM;
	
	if(folder->VarDict["infilQ"] < 0.0 )
		folder->VarDict["infilQ"] = 0.00001;

}

void SoilInfiltrationPowDecayCalc::simpsonIntegration(double IP, double F, double& IR, DataFolder *folder, Inputs * input)
{

	int N =1;
	double H =F - IP;
	double T1 = 0.0;
	double T2 =0.0;
	double S1 = 0.0;
	double S2 = 0.0;
	double X =0.0;
	double P ;
	double fF = 1/folder->ParamDict["m"];
	double CSSF = (folder->ParamDict["WFS_m"]*folder->ParamDict["DWSMI_m"]);



	bool finish = false;
	T1 = H*( -log(1-IP*fF)/(CSSF+IP) - log(1-F*fF)/(CSSF+F) )/2,0 ;
	S1 = T1;

	do
	{
		P =0.0;
		for(int k=0; k<=N-1; k++)
		{
			X = IP + (k+0.5)*H ;
			P +=-log(1-X*fF)/(CSSF+X);
		}

		T2 = (T1+H*P)/2.0;
		S2 = (4*T2 -T1)/3.0;

		if(fabs(S2-S1) >= 0.00001)
		{
			T1 = T2;
			N += N;
			H = 0.5*H;
			S1 = S2;
			finish = false;
		}
		else
		{
			IR = S2;
			finish = true;
		}
	} while(!finish);




}