#include "ShortVegInterceptionCalc.h"

void ShortVegInterceptionCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["shortVegThruFlowW"] = 0.0;					   //initial these parameters for each time step
	folder->VarDict["shortVegThruFlowSWE"] = 0.0;
	folder->VarDict["shortVegIntW"] = 0.0;
	folder->VarDict["shortVegIntSWE"] = 0.0;

	if (input->Tair[timeStep] > 32.0)
		shortVegInterceptionRain(input, folder, timeStep);
	else
		shortVegInterceptionSnow(input, folder, timeStep);

	shortVegUnloadSnow(input, folder, timeStep);
	
	//Only allow for snowmelt by radiation if snow storage on shortvegetation 
	if (folder->VarDict["shortVegStoredSWE"]>0.0)
		shortVegSnowMeltbyRn(input, folder, timeStep);

	// New function added to keep updating/storage in one place
	storeValues(input, folder, timeStep);

}


void ShortVegInterceptionCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{

	// Updating the combined throughfall to include the water throughfall and snow-water equivalent throughfall
	folder->VarDict["shortVegThroughQ"] = folder->VarDict["shortVegThruFlowW"] + folder->VarDict["shortVegThruFlowSWE"];
	// Updating the combined interception to include the water int and snow-water equivalent int
	folder->VarDict["shortVegInt"] = folder->VarDict["shortVegIntW"] + folder->VarDict["shortVegIntSWE"];

	// Updating total variables
	folder->VarDict["totalShortVegIntW"] += folder->VarDict["shortVegIntW"];
	folder->VarDict["totalShortVegInterceptionSnowWE_m"] += folder->VarDict["shortVegIntSWE"];
	folder->VarDict["totalShortVegInt"] += folder->VarDict["shortVegInt"];
	folder->VarDict["totalShortVegThroughW"] += folder->VarDict["shortVegThruFlowW"];
	folder->VarDict["totalShortVegThroughSWE"] += folder->VarDict["shortVegThruFlowSWE"];
	folder->VarDict["totalShortVegThroughQ"] += folder->VarDict["shortVegThroughQ"];

	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		// Updating vector variables
		folder->VarVecDict["shortVegIntW"].push_back(folder->VarDict["shortVegIntW"]);
		folder->VarVecDict["shortVegIntSWE"].push_back(folder->VarDict["shortVegIntSWE"]);
		folder->VarVecDict["shortVegInt"].push_back(folder->VarDict["shortVegInt"]);
		folder->VarVecDict["shortVegThroughW"].push_back(folder->VarDict["shortVegThruFlowW"]);
		folder->VarVecDict["shortVegThroughSWE"].push_back(folder->VarDict["shortVegThruFlowSWE"]);
		folder->VarVecDict["shortVegThroughQ"].push_back(folder->VarDict["shortVegThroughQ"]);
	}
}


void ShortVegInterceptionCalc::shortVegUnloadSnow(Inputs *input, DataFolder *folder, int timeStep)
{
	double shortVegUnloadSWE = 0.0231 * input->WindSp_mps[timeStep] * folder->VarDict["shortVegStoredSWE"];
	
	if (shortVegUnloadSWE > folder->VarDict["shortVegStoredSWE"])
	{
		shortVegUnloadSWE = folder->VarDict["shortVegStoredSWE"];
	}

	folder->VarDict["shortVegStoredSWE"] = folder->VarDict["shortVegStoredSWE"] - shortVegUnloadSWE;
	folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + shortVegUnloadSWE;

}

void ShortVegInterceptionCalc::shortVegSnowMeltbyRn(Inputs *input, DataFolder *folder, int timeStep)
{
	double Mn = 0.000011 * input->RadiationNet_Wpm2[timeStep] * (1 - 0.85);     // assume the snow surface albedo is 0.85
	double Ml = 0.0212 * 0.0010583 * (input->Tair[timeStep] - 32.0) - 0.84 * 0.0010583;
	if (Ml < 0)
		Ml=0;
	double RnMeltedSWE = Mn+Ml; // it is snow water equivalent

	//double RnMeltedSnow=WaterDToSnowD(W,RnMeltedSnowW,i);
	if (RnMeltedSWE > folder->VarDict["shortVegStoredSWE"])
	{
		RnMeltedSWE = folder->VarDict["shortVegStoredSWE"];
	}
	
	folder->VarDict["shortVegStoredSWE"] = folder->VarDict["shortVegStoredSWE"] - RnMeltedSWE;
	folder->VarDict["shortVegStoredW"] = folder->VarDict["shortVegStoredW"] + RnMeltedSWE;
	
	if (folder->VarDict["shortVegStoredW"] >folder->VarDict["storMaxShortVeg"])  
	{   
		folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + folder->VarDict["shortVegStoredSWE"];
		folder->VarDict["shortVegStoredSWE"] = 0.0;
		folder->VarDict["shortVegThruFlowW"] = folder->VarDict["shortVegThruFlowW"] + (folder->VarDict["shortVegStoredW"] - folder->VarDict["storMaxShortVeg"]);
		folder->VarDict["shortVegStoredW"] = folder->VarDict["storMaxShortVeg"];
	}
	folder->VarDict["shortVegStoredSWE"] = (folder->VarDict["shortVegStoredSWE"] >= 0.0) ? folder->VarDict["shortVegStoredSWE"] : 0.0;
	folder->VarDict["shortVegStoredW"] = (folder->VarDict["shortVegStoredW"] >= 0.0) ? folder->VarDict["shortVegStoredW"] : 0.0;
}



void ShortVegInterceptionCalc::shortVegInterceptionRain(Inputs *input, DataFolder *folder, int timeStep)
{
	double tempShortVegLAI = input->ShortVegLAI[timeStep];
	if (folder->ParamStringDict["Type"]  !="BulkArea")
	{
		double factor = folder->ParamDict["MaxShortVegLAI"] / 2.2;
		tempShortVegLAI *= factor;
	}
	folder->VarDict["storMaxShortVeg"] = (0.15 * 0.001 * tempShortVegLAI);     //0.001 convert mm -> m for AveShortVegSL, here AveShortVegSL is mm
	double fShortVegTQ  =  exp(-0.3 *  tempShortVegLAI);
	folder->VarDict["shortVegThruFlowW"] = fShortVegTQ * input->Precip_mpdt[timeStep];
	folder->VarDict["shortVegIntW"] = (1-fShortVegTQ) * input->Precip_mpdt[timeStep];

	// Reza, making a new variable for the rainfall in every time step to use in the GI routine
	folder->VarDict["rainfall"] = input->Precip_mpdt[timeStep];

	double MeltedSnowbyRain = 0.0129 * folder->VarDict["shortVegIntW"] * (input->Tair[timeStep] - 32);

	if (MeltedSnowbyRain > folder->VarDict["shortVegStoredSWE"])      // melted snow cannot be larger than the stored snow
		MeltedSnowbyRain = folder->VarDict["shortVegStoredSWE"];

	folder->VarDict["shortVegStoredSWE"] = folder->VarDict["shortVegStoredSWE"] - MeltedSnowbyRain;  // update the stored snow (water equivalent)

	double tempShortVegStoredW = folder->VarDict["shortVegStoredW"] + folder->VarDict["shortVegIntW"] + MeltedSnowbyRain;

	if (tempShortVegStoredW > folder->VarDict["storMaxShortVeg"])
	{
		folder->VarDict["shortVegIntW"] = folder->VarDict["storMaxShortVeg"] - folder->VarDict["shortVegStoredW"] - MeltedSnowbyRain;
		folder->VarDict["shortVegThruFlowW"] = folder->VarDict["shortVegThruFlowW"] + (tempShortVegStoredW - folder->VarDict["storMaxShortVeg"]);
		folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + folder->VarDict["shortVegStoredSWE"];
		folder->VarDict["shortVegStoredSWE"] = 0.0;
		tempShortVegStoredW = folder->VarDict["storMaxShortVeg"];
	}

	folder->VarDict["shortVegStoredW"] = tempShortVegStoredW;
	
	if(folder->VarDict["shortVegIntW"] < 0)
		folder->VarDict["shortVegIntW"] = 0.0;
}

void ShortVegInterceptionCalc::shortVegInterceptionSnow(Inputs *input, DataFolder *folder, int timeStep)
{
	double tempShortVegLAI = input->ShortVegLAI[timeStep];
	if (folder->ParamStringDict["Type"]  !="BulkArea")
	{
		double factor = folder->ParamDict["MaxShortVegLAI"] / 2.2;
		tempShortVegLAI *= factor;
	}
	folder->VarDict["storMaxShortVeg"] = 0.15 * 0.001 * tempShortVegLAI;   
	
	double Tair = 5.0/9.0 * (input->Tair[timeStep] - 32); // should be 0C
	double SnowDensity = 67.92 + 51.25 * exp(Tair/2.59);
	double S = 6.0 * (0.27 + 46.0/SnowDensity)/SnowDensity;    // 6.0 can be changed
	double storMaxShortVegSnow = S * tempShortVegLAI;
	double storMaxShortVegSWE = storMaxShortVegSnow * (SnowDensity/1000);
	//Eq 2a & Eq 2b from Wang, Endreny, Nowak (2008, JAWRA) based on Dijk and Bruijnzeel (2001) for free throughfall
	//Wang et al. (2008) after Eq 2b assign extinction coefficient k = 0.3 to short vegetation
	double fShortVegTQ = (1-(1-exp(-0.3 * tempShortVegLAI)));
	double SnowP = input->Precip_mpdt[timeStep];
	
	folder->VarDict["shortVegThruFlowSWE"] = fShortVegTQ * SnowP;   //SnowP is the Snow water equivalent depth
	folder->VarDict["shortVegIntSWE"] = (1-fShortVegTQ) * SnowP; // need to convert precip to snow depth first

	double  tempShortVegStoredSWE = folder->VarDict["shortVegStoredSWE"] + folder->VarDict["shortVegIntSWE"];
	if (tempShortVegStoredSWE > storMaxShortVegSWE)
	{	
		folder->VarDict["shortVegIntSWE"] = storMaxShortVegSWE - folder->VarDict["shortVegStoredSWE"];
		folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + (tempShortVegStoredSWE - storMaxShortVegSWE);
		tempShortVegStoredSWE = storMaxShortVegSWE;
	} 

	folder->VarDict["shortVegStoredSWE"] = tempShortVegStoredSWE;


	if(folder->VarDict["shortVegIntSWE"] < 0)
		folder->VarDict["shortVegIntSWE"] = 0.0;
}
