﻿#include "SnowMeltOpenAreaCalc.h"

/**************************************************************************** 
Function Name: SnowMeltOpenAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep) 
Purpose: To calculate:
			- the amount of melted snow in an open area (no tree or shortveg cover)
			- the total amount of water in an open area (o tree or shortveg cover)
		 Updates:
			- the amount of snow on the open area (leftover snow + new snow)
			- the amount of water on the open area (rain + metled snow)

Requires:		int timeStep - the timestep value
				DataFolder *folder - the key number for the folder being processed
Returns:		Void

Reference:   1. Yang, Yang, Theodore A. Endreny, and David J. Nowak, 2011. 
                  iTree-Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model. 
                  Journal of the American Water Resources Association (JAWRA) 47(6):1211–1218.
                  DOI: 10.1111/j.1752-1688.2011.00564.x

Comments:	Introduced by Yang Yang in iTree V5, to include snow processes, adapts and uses equations from
            Roesch et al. (2001) and USACE (1958, 1998)
			The differences in in the snow melt equations (between open area, under tree, under shortveg) 
			are explained in Yang's paper.

Author(s):		Jun Wang, jwang06@mailbox.syr.edu, V4/V4.5
				Yang Yang, yyang31@syr.edu, V5
				 	
				Thomas Taggart, traggatmot@gmail.com
						
*****************************************************************************/


void SnowMeltOpenAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	meltsnowOpen(input, folder, timeStep);
	storeValues(input, folder, timeStep);
}

void SnowMeltOpenAreaCalc::meltsnowOpen(Inputs *input, DataFolder *folder, int timeStep)
{
	double Tair = input->Tair[timeStep]; 
	double P = input->Precip_mpdt[timeStep];   
	double RadiationNet_Wpm2 = input->RadiationNet_Wpm2[timeStep];		
	double Tdew = input->Tdew[timeStep];  
	double UV = input->WindSp_mps[timeStep];  
	double Kp = 1.0;
	double Za = 2.0;
	double Zb = 2.0;

	// The following Snow equations are applied by Yang, and originate from:
	// USACE, 1956. Snow Hydrology: Summary Report of the Snow Investigations. 
	// US Army Corps of Engineers North Pacific Division Report NTIS PB-151660, Portland, Oregon.
	// USACE, 1998. Engineering and Design: Runoff From Snowmelt. 
	// US Army Corps of Engineers Report Engineer Manual 1110-2-1406, Washington, D.C.
	
	// initializing the snow melt by rain to 0.0
	double Mp = 0.0;

	// Kp, Za, Zb are initialized in the SnowMeltUnderTreeRepo constructor - Kp(0.8), Za(2.0), Zb(2.0)
	// kp is the ratio of atmospheric pressure at the location to the atmospheric pressure at the sea level
	// za (m) is the height at which the temperature and the air vapor pressure are measured
	// zb (m) is the height of wind velocity measurement

	// Mh (m/hr) = the snow melt by sensible heat = 0.0000044*Kp*(Za*Zb)^(-1/6) * (Tair - 32) * UV
	// The original equation used this constant: 0.00000792.  This equation applies the fraction (5.0/9.0), from conversion of F to C, to get 0.0000044
	double Mh = 0.0000044 * Kp * pow(Za * Zb, -(1.0/6.0)) * (Tair - 32) * UV;
	// Me (m/hr) = snowmelt by latent heat.  0.0000124 has been arrived by applying (5.0/9.0) to 0.00000223
	double Me = 0.0000124 * pow(Za * Zb, -(1.0/6.0)) * (Tdew - 32) * UV;
	// Mn (m⁄hr) =  the snowmelt by net radiation of both shortwave radiation
	double Mn = 0.000011 * RadiationNet_Wpm2 * (1 - 0.85);   //0.85 is the snow surface albedo
	// Ml (m/hr) = the snow melt 
	double Ml = 0.0;

		
	if (P > 0)  // For cloudy sky - Having input precipitation assumes cloudy sky
		Ml = 0.0212 * 0.0010583 * (Tair - 32.0);   // Melt by long wave radiation on cloudy day
	else   // For clear sky - Having no precip to input assumes clear sky
		Ml = 0.0212 * 0.0010583 * (Tair - 32.0) - 0.84 * 0.0010583;   // Melt by longwave radiation on clear day
	// Completing the math, we get the original numners: Ml = 0.00002243596*(Tair-32)(5.0/9.0) - 0.000888972     

	// Mg (m⁄hr) is the snowmelt by the ground heat
	double Mg = 0.0000277;    
	
	double RainOpen = 0.0;    // Initializing the rainfall (Precuip) to 0.0

	// If melt by long wave radiaiton < 0, set it to 0
	if (Ml < 0)
	Ml = 0;

	// The total melt by radiaton  = the shortwave melt plus the long wave melt
	Mn = Mn + Ml;

	// If the temperature is above 32F, we are receiving rain
	if (Tair > 32.0)
	{
		RainOpen = P;
		Mp = 0.0129 * RainOpen * (Tair - 32);    // Update the snow melted by rain
	}
	
	// The temp is less than 32F, we are receiving snow
	if(Tair <= 32.0)
	{
		RainOpen = 0.0;
		// Update the snowOpen with the additional snow fall
		double snow = P + folder->VarDict["snowOpen"];
		folder->VarDict["snowOpen"] = snow;
	}


	// Modify the melted OpenSnow amount = melted snow from all the different melt pathways
	folder->VarDict["snowMeltOpen"] = Mn + Mp + Mg + Mh + Me;

	// If the metled snow is somehow less than 0 (meaning snow was gained through the melt processes) set it to 0
	if (folder->VarDict["snowMeltOpen"] < 0.0)
		folder->VarDict["snowMeltOpen"] = 0.0;

	// If the melted snow is > snowOpen, set it equal to snowOpen
	if(folder->VarDict["snowMeltOpen"] > folder->VarDict["snowOpen"])
		folder->VarDict["snowMeltOpen"] = folder->VarDict["snowOpen"];

	// Update the snowOpen to reflect the loss of the melted snow
	folder->VarDict["snowOpen"] = folder->VarDict["snowOpen"] - folder->VarDict["snowMeltOpen"];
			
	// Read in the potential Evap for Snow		
	double SnowEP = input->PESnow[timeStep];
	
	// The amount of snow that could sublimate is > snow available, set equal to snowOpen (snow available)
	if (SnowEP > folder->VarDict["snowOpen"])
		SnowEP = folder->VarDict["snowOpen"];
	
	// Update snowOpen area to reflect the loss of the sublimated snow
	double snowLeft = folder->VarDict["snowOpen"] - SnowEP;
	folder->VarDict["snowOpen"] = snowLeft;

	// Update the TotalWaterFromOpenArea (this folder, this timestep) to be = (RainOpen (Precip_mpdt falling on the open area) + MeltedSOpen)
	folder->VarDict["waterOpenArea"] = folder->VarDict["snowMeltOpen"] + RainOpen;

	// updating the snow evap under tree
	folder->VarDict["snowEvapOpen"] = SnowEP;

}


void SnowMeltOpenAreaCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["totalSnowEvapOpen_mm"] += folder->VarDict["snowEvapOpen"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		// Updating vectors
		folder->VarVecDict["snowEvapOpen"].emplace_back(folder->VarDict["snowEvapOpen"]);
		folder->VarVecDict["snowMeltOpen"].emplace_back(folder->VarDict["snowMeltOpen"]);
		folder->VarVecDict["snowOpen"].emplace_back(folder->VarDict["snowOpen"]);
	}
}
