#include "SnowMeltUnderTreeCalc.h"

/**************************************************************************** 
Function Name: SnowMeltUnderTreeCalc::calculate(Inputs *input, DataFolder *folder, int timeStep) 
Purpose: To calculate:
			- the amount of melted snow under the tree
			- the total amount of water under the tree
		 Updates:
			- the amount of snow under the tree
Requires:		int timeStep - the timestep value
				DataFolder *folder - the key number for the folder being processed
Returns:		Void

Reference:   1. Yang, Yang, Theodore A. Endreny, and David J. Nowak, 2011. 
                  iTree-Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model. 
                  Journal of the American Water Resources Association (JAWRA) 47(6):12111218.
                  DOI: 10.1111/j.1752-1688.2011.00564.x

Comments:	Introduced by Yang Yang in iTree V5, to include snow processes, adapts and uses equations from
            Roesch et al. (2001) and USACE (1958, 1998)

Author(s):		Jun Wang, jwang06@mailbox.syr.edu, V4/V4.5
				Yang Yang, yyang31@syr.edu, V5
				 	
				Thomas Taggart, traggatmot@gmail.com
						
*****************************************************************************/


void SnowMeltUnderTreeCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	meltsnowUnderTree(input, folder, timeStep);
	storeValues(input, folder, timeStep);
}

void SnowMeltUnderTreeCalc::meltsnowUnderTree(Inputs *input, DataFolder *folder, int timeStep)
{

	// Update the snowUnderTree to include the SWE throughfall
	double temp = folder->VarDict["snowUnderTree"] + folder->VarDict["treeThruFlowSWE"];
	folder->VarDict["snowUnderTree"] = temp;


	// Placing the input air temp into Tair
	double Tair = input->Tair[timeStep];  
	// Placing the tree thoughfall into P (this is folder (instead of timestep) because it is stored for each timestep once in each folder - this is not the vector that stores every timestep)
	double P = folder->VarDict["treeThruFlowW"];  
	// Placing the input solar radiation into RadiationNet_Wpm2
	double RadiationNet_Wpm2 = input->RadiationNet_Wpm2[timeStep];		
	// Placing the input dew Temperature into Tdew 
	double Tdew = input->Tdew[timeStep]; 
	// Placing the input windspeed into UV
	double UV = input->WindSp_mps[timeStep];                     

	// Kp, Za, Zb are initialized in the SnowMeltUnderTreeRepo constructor - Kp(0.8), Za(2.0), Zb(2.0)
	// kp is the ratio of atmospheric pressure at the location to the atmospheric pressure at the sea level
	// za (m) is the height at which the temperature and the air vapor pressure are measured
	// zb (m) is the height of wind velocity measurement

	double Kp = 0.8;
	double Za = 2.0;
	double Zb = 2.0;

	// Mp = snow melted by rain
	double Mp = 0.0129 * P * (Tair - 32);
	// Ms (m/hr) = snow melt by short wave radiation
	double Ms = 0.0000317;
	// Ml (m/hr) = snow melt by long wave radiation 
	double Ml = 0.0000307 * (Tair - 32);
	// Mh (m/hr) = the snow melt by sensible heat = 0.0000044*Kp*(Za*Zb)^(-1/6) * (Tair - 32) * UV
	// The original equation used this constant: 0.00000792.  This equation applies the fraction (5.0/9.0), from conversion of F to C, to get 0.0000044
	double Mh = 0.0000044 * Kp * pow(Za*Zb, -(1.0/6.0)) * (Tair - 32) * UV;
	// Me (m/hr) = snowmelt by latent heat.  0.0000124 has been arrived by applying (5.0/9.0) to 0.00000223
	double Me = 0.0000124 * pow(Za*Zb, -(1.0/6.0)) * (Tdew - 32) * UV;
	// Mg (m/hr) = snow melt from the ground heat
	double Mg = 0.000021;
	
	// Modify the MeltUnderTree amount = melted snow from all the different melt pathways
	folder->VarDict["snowMeltUnderTree"] = Ms + Ml + Mp + Mg + 0.3*(Mh + Me);       
	//0.3 is the wind coefficient and can be changed to other value in range 0.3 to 1.0

	// If the metled snow is somehow less than 0 (meaning snow was gained through the melt processes) set it to 0
	if (folder->VarDict["snowMeltUnderTree"] < 0)
		folder->VarDict["snowMeltUnderTree"] = 0.0;

	// If the melted snow is > snowOpen, set it equal to snowOpen
	if (folder->VarDict["snowMeltUnderTree"] > folder->VarDict["snowUnderTree"])
		folder->VarDict["snowMeltUnderTree"] = folder->VarDict["snowUnderTree"];

	// Update the snowUnderTree to reflect the loss of the melted snow
	temp = folder->VarDict["snowUnderTree"] - folder->VarDict["snowMeltUnderTree"];
	folder->VarDict["snowUnderTree"] = temp;

	// Read in the potential Evap for Snow
	double SnowEP = input->PESnow[timeStep];

	// The amount of snow that could sublimate is > snow available, set equal to snowUnderTree (snow available)
	if (SnowEP > folder->VarDict["snowUnderTree"])
		SnowEP = folder->VarDict["snowUnderTree"];

	// Update snowUnderTree area to reflect the loss of the sublimated snow
	temp = folder->VarDict["snowUnderTree"] - SnowEP;
	folder->VarDict["snowUnderTree"] = temp;

	// Update the waterUnderTree (this folder, this timestep) to be = (RainOpen (Precip_mpdt falling on the open area) + snowMeltUnderTree)
	folder->VarDict["waterUnderTree"] = folder->VarDict["snowMeltUnderTree"] + P;

	// updating the snow evap under tree
	folder->VarDict["snowEvapUnderTree"] = SnowEP;
}



void SnowMeltUnderTreeCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["totalSnowEvapUnderTree_mm"] += folder->VarDict["snowEvapUnderTree"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		// Updating vectors
		folder->VarVecDict["snowEvapUnderTree"].emplace_back(folder->VarDict["snowEvapUnderTree"]);
		folder->VarVecDict["snowMeltUnderTree"].emplace_back(folder->VarDict["snowMeltUnderTree"]);
		folder->VarVecDict["snowUnderTree"].emplace_back(folder->VarDict["snowUnderTree"]);
	}
}

