#include "StoreTimeStepTotalsCalc.h"

/****************************************************************************
Function Name: SnowMeltUnderTreeCalc::calculate(int timeStep, std::vector<DataFolder*> &stafolders)
Purpose: To calculate:
- the amount of melted snow under the tree
- the total amount of water under the tree
Updates:
- the amount of snow under the tree
Requires:		int timeStep - the timestep value
std::vector<DataFolder*> &stafolders - the key number for the folder being processed
Returns:		Void

Reference:   1. Yang, Yang, Theodore A. Endreny, and David J. Nowak, 2011.
iTree-Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model.
Journal of the American Water Resources Association (JAWRA) 47(6):12111218.
DOI: 10.1111/j.1752-1688.2011.00564.x

Comments:	Introduced by Yang Yang in iTree V5, to include snow processes, adapts and uses equations from
Roesch et al. (2001) and USACE (1958, 1998)

Author(s):		Jun Wang, jwang06@mailbox.syr.edu, V4/V4.5
Yang Yang, yyang31@syr.edu, V5
 
Thomas Taggart, traggatmot@gmail.com

*****************************************************************************/


void StoreTimeStepTotalsCalc::calculate(Inputs *input, int timeStep, std::vector<std::vector<DataFolder*> > &DataDrawers)
{
	storeorganizerValues(input, timeStep, DataDrawers);
}


void StoreTimeStepTotalsCalc::storeorganizerValues(Inputs *input, int timeStep, std::vector<std::vector<DataFolder*> > &DataDrawers)
{
	double area = 0.0;              // area this folder makes up - THIS NEEDS TO CONSIDER THE CONTRIBUTING AREA OF EACH GI folder
	double fArea = 0.0;             // fraction of the watershed this folder makes up
	double watershedArea = input->SimulationNumericalParams["CatchmentArea_m2"];

	double totalTreeInterceptionRain_m = 0.0;
	double totalTreeInterceptionSnowWE_m = 0.0;
	double totalShortVegInterceptionRain_m = 0.0;
	double totalShortVegInterceptionSnowWE_m = 0.0;
	double totalTreeEvap_m = 0.0;
	double totalShortVegEvap_m = 0.0;
	double totalTreeThruQ = 0.0;
	double totalShortVegThruQ = 0.0;
	double totalsnowOpen = 0.0;
	double totalsnowUnderTree = 0.0;
	double totalsnowUnderShortVeg = 0.0;
	double totalSnowEvapOpen_m = 0.0;
	double totalSnowEvapUnderTree_m = 0.0;
	double totalSnowEvapUnderShortVeg_m = 0.0;
	double totalWaterOnPerviousArea_m = 0.0;
	double totalWaterOnImperviousArea_m = 0.0;
	double totalWaterToSoil = 0.0;
	double totalImpDepEvap_m = 0.0;
	double totalImpDepStor = 0.0;
	double totalImpFlowToSoil_m = 0.0;
	double totalPerDepEvap_m = 0.0;
	double totalPerDepStor = 0.0;
	double totalPerFlowToInfil = 0.0;
	double totalInfiltration_m = 0.0;
	double totalDCIAFlowToOutlet_m = 0.0;
	double totalInfilExQ = 0.0;
	double totalSatExQ = 0.0;
	double totalQQi = 0.0;
	double totalBQi = 0.0;
	double totalPAQi = 0.0;
	double totalDCIAQi = 0.0;


	for (int i = 0; i < DataDrawers.size(); ++i)
	{
		for (int j = 0; j < DataDrawers[i].size(); ++j)
		{
			if (DataDrawers[i][j]->ParamStringDict["Type"]  != "NODATA")
			{
				DataFolder *folder = DataDrawers[i][j];
				area = folder->ParamDict["Area"];
				fArea = area / watershedArea;


				totalTreeInterceptionRain_m += folder->VarDict["treeIntW"] * fArea;
				totalTreeInterceptionSnowWE_m += folder->VarDict["treeIntSWE"] * fArea;

				totalShortVegInterceptionRain_m += folder->VarDict["shortVegIntW"] * fArea;
				totalShortVegInterceptionSnowWE_m += folder->VarDict["shortVegIntSWE"] * fArea;

				totalTreeEvap_m += folder->VarDict["treeEvap"] * fArea;
				totalShortVegEvap_m += folder->VarDict["shortVegEvap"] * fArea;
				
				totalTreeThruQ += folder->VarDict["treeThruFlow"] * fArea;
				totalShortVegThruQ += folder->VarDict["shortVegThroughQ"] * fArea;

				totalsnowOpen += folder->VarDict["snowOpen"] * fArea;
				totalsnowUnderTree += folder->VarDict["snowUnderTree"] * fArea;
				totalsnowUnderShortVeg += folder->VarDict["snowUnderShortVeg"] * fArea;

				totalSnowEvapOpen_m += folder->VarDict["snowEvapOpen"] * fArea;
				totalSnowEvapUnderTree_m += folder->VarDict["snowEvapUnderTree"] * fArea;
				totalSnowEvapUnderShortVeg_m += folder->VarDict["snowEvapUnderShortVeg"] * fArea;

				totalWaterOnPerviousArea_m += folder->VarDict["waterOnPerviousArea"] * fArea;
				totalWaterOnImperviousArea_m += folder->VarDict["waterOnImperviousArea"] * fArea;

				totalImpDepEvap_m += folder->VarDict["impEvap_m"] * fArea;
				totalImpDepStor += folder->VarDict["impDepStor_m"] * fArea;
				totalImpFlowToSoil_m += folder->VarDict["impRunoffToSoil_m"] * fArea;

				totalPerDepEvap_m += folder->VarDict["perDepEvap"] * fArea;
				totalPerDepStor += folder->VarDict["perDepStor"] * fArea;
				totalPerFlowToInfil += folder->VarDict["runoffToInfil"] * fArea;

				totalWaterToSoil += folder->VarDict["waterToPerDep"] * fArea;

				totalDCIAFlowToOutlet_m += folder->VarDict["impRunoffToOutlet_m"] * fArea;

				totalInfilExQ += folder->VarDict["infilExRunoff"] * fArea;
				totalSatExQ += folder->VarDict["satExRunoff"] * fArea;
			}
		}

	}

	input->RepoVecDict["totalTreeIntRain_TS"].push_back(totalTreeInterceptionRain_m);
	input->RepoDict["totalTreeInterceptionRain_m"] += totalTreeInterceptionRain_m;

	input->RepoVecDict["totalTreeIntSWE_TS"].push_back(totalTreeInterceptionSnowWE_m);
	input->RepoDict["totalTreeInterceptionSnowWE_m"] += totalTreeInterceptionSnowWE_m;

	input->RepoVecDict["totalShortVegIntRain_TS"].push_back(totalShortVegInterceptionRain_m);
	input->RepoDict["totalShortVegInterceptionRain_m"] += totalShortVegInterceptionRain_m;

	input->RepoVecDict["totalShortVegIntSWE_TS"].push_back(totalShortVegInterceptionSnowWE_m);
	input->RepoDict["totalShortVegInterceptionSnowWE_m"] += totalShortVegInterceptionSnowWE_m;

	input->RepoVecDict["totalTreeThruQ_TS"].push_back(totalTreeThruQ);
	input->RepoDict["totalTreeThruFlow_m"] += totalTreeThruQ;

	input->RepoVecDict["totalShortVegThruQ_TS"].push_back(totalShortVegThruQ);
	input->RepoDict["totalShortVegThruFlow_m"] += totalShortVegThruQ;

	input->RepoVecDict["totalTreeEvap_TS"].push_back(totalTreeEvap_m);
	input->RepoDict["totalTreeEvap_m"] += totalTreeEvap_m;

	input->RepoVecDict["totalShortVegEvap_TS"].push_back(totalShortVegEvap_m);
	input->RepoDict["totalShortVegEvap_m"] += totalShortVegEvap_m;

	input->RepoVecDict["totalSnowOpen_TS_m"].push_back(totalsnowOpen);
	input->RepoDict["totalSnowOpen"] += totalsnowOpen;

	input->RepoVecDict["totalSnowUnderTree_TS_m"].push_back(totalsnowUnderTree);
	input->RepoDict["totalSnowUnderTree"] += totalsnowUnderTree;

	input->RepoVecDict["totalSnowUnderShortVeg_TS_m"].push_back(totalsnowUnderShortVeg);
	input->RepoDict["totalSnowUnderShortVeg"] += totalsnowUnderShortVeg;

	input->RepoVecDict["totalSnowEvapOpen_TS"].push_back(totalSnowEvapOpen_m);
	input->RepoDict["totalSnowEvapOpen_m"] += totalSnowEvapOpen_m;

	input->RepoVecDict["totalSnowEvapUnderTree_TS"].push_back(totalSnowEvapUnderTree_m);
	input->RepoDict["totalSnowEvapUnderTree_m"] += totalSnowEvapUnderTree_m;

	input->RepoVecDict["totalSnowEvapUnderShortVeg_TS"].push_back(totalSnowEvapUnderShortVeg_m);
	input->RepoDict["totalSnowEvapUnderShortVeg_m"] += totalSnowEvapUnderShortVeg_m;

	input->RepoVecDict["totalWaterOnPerviousArea_TS"].push_back(totalWaterOnPerviousArea_m);
	input->RepoDict["totalWaterOnPerviousArea_m"] += totalWaterOnPerviousArea_m;

	input->RepoVecDict["totalWaterOnImperviousArea_TS"].push_back(totalWaterOnImperviousArea_m);
	input->RepoDict["totalWaterOnImperviousArea_m"] += totalWaterOnImperviousArea_m;

	input->RepoVecDict["totalImpDepEvap_TS"].push_back(totalImpDepEvap_m);
	input->RepoDict["totalImpDepEvap_m"] += totalImpDepEvap_m;

	input->RepoVecDict["totalImpDepStor_TS_m"].push_back(totalImpDepStor);
	input->RepoDict["totalImpDepStor"] += totalImpDepStor;

	input->RepoVecDict["totalImpFlowToSoil_TS"].push_back(totalImpFlowToSoil_m);
	input->RepoDict["totalImpFlowToSoil_m"] += totalImpFlowToSoil_m;

	input->RepoVecDict["totalPerDepEvap_TS"].push_back(totalPerDepEvap_m);
	input->RepoDict["totalPerDepEvap_m"] += totalPerDepEvap_m;

	input->RepoVecDict["totalPerDepStore_TS_m"].push_back(totalPerDepStor);
	input->RepoDict["totalPerDepStor"] += totalPerDepStor;

	input->RepoVecDict["totalPerFlowToInfil_TS"].push_back(totalPerFlowToInfil);
	input->RepoDict["totalPerFlowToInfil_m"] += totalPerFlowToInfil;

	input->RepoVecDict["totalWaterToSoil_TS"].push_back(totalWaterToSoil);
	input->RepoDict["totalWaterToSoil_m"] += totalWaterToSoil;

	input->RepoVecDict["totalFlowToDCIA_TS"].push_back(totalDCIAFlowToOutlet_m);
	input->RepoDict["totalDCIAFlowToOutlet_m"] += totalDCIAFlowToOutlet_m;

	input->RepoVecDict["totalInfilExQ_TS"].push_back(totalInfilExQ);
	input->RepoDict["totalInfilExQ"] += totalInfilExQ;

	input->RepoVecDict["totalSatExQ_TS"].push_back(totalSatExQ);
	input->RepoDict["totalSatExQ"] += totalSatExQ;
}


