/*	
 *  InterpolationCal.cpp - Provdes methods to interpolate hourly temperatures.
 */

#include "ICTCInterpolater.h"

const std::string ICTCInterpolator::IC0TC0_AIR_TEMP = "IC0TC0AirTemp";
const std::string ICTCInterpolator::IC0TC0_DEW_TEMP = "IC0TC0DewTemp";
const std::string ICTCInterpolator::IC0TC100_AIR_TEMP = "IC0TC100AirTemp";
const std::string ICTCInterpolator::IC0TC100_DEW_TEMP = "IC0TC100DewTemp";
const std::string ICTCInterpolator::IC100TC0_AIR_TEMP = "IC100TC0AirTemp";
const std::string ICTCInterpolator::IC100TC0_DEW_TEMP = "IC100TC0DewTemp";

const double ICTCInterpolator::NORMALIZATION_FACTOR = 100.0;

void readEdgeCaseData(Inputs* inputs, map<std::string, map<std::string, double>>* edgeCaseData);

/*
 * Initializes the fields used to interpolate hourly temperature. Code is 
 * borrowed from interpolate.cs of HeatMetricCalculator.
 */
ICTCInterpolator::ICTCInterpolator()
{
    // used to adjust the values of the arrays to the correct range
    double normalizationFactor = 100.0;
}

/*
 * Computers the icv interpolation factor for the given ICV value.
 *
 * @param icValue the IC value for the interpolation
 */
double ICTCInterpolator::computeICVInterpolationFactor(int icValue)
{
    return icValue / NORMALIZATION_FACTOR;
}

/*
 * Computers the tc interpolation factor for the given TC value.
 *
 * @param icValue the IC value for the interpolation
 * @param tcValue the TC value for the interpolation
 */
double ICTCInterpolator::computeTCInterpolationFactor(int icValue, int tcValue)
{
    return tcValue / NORMALIZATION_FACTOR;
}

/*
 * Computers the ic interpolation factor for the given IC value.
 *
 * @param icValue the IC value for the interpolation
 * @param tcValue the TC value for the interpolation
 */
double ICTCInterpolator::computeICInterpolationFactor(int icValue, int tcValue)
{
    double icv = icValue / NORMALIZATION_FACTOR;
    double tcv = tcValue / NORMALIZATION_FACTOR;
    return (icv + tcv > 1) ? 1 - tcv : icv;
}

/*
 * Computers the sv interpolation factor for the given TC value.
 *
 * @param icValue the IC value for the interpolation
 * @param tcValue the TC value for the interpolation
 */
double ICTCInterpolator::computeSVInterpolationFactor(int icValue, int tcValue)
{
    double tcv = tcValue / NORMALIZATION_FACTOR;
    double icv = icValue / NORMALIZATION_FACTOR;
    return (1 - tcv - icv) < 0 ? 0 : 1 - tcv - icv;
}


/*
 * Interpolates the hourly temperature for TCXTCX using the edge case values svt, ict,
 * and tct. Code is borrowed from interpolate.cs of HeatMetricCalculator.
 *
 * @param svt the IC0TC0 hourly temperature
 * @param ict the IC100TC0 hourly temperature
 * @param tct the IC0TC100 hourly temperature
 * @param icValue the ICX value for the interpolation
 * @param tcValue the TCX value for the interpolation
 */
double ICTCInterpolator::interpolateHourlyTemperature(double svt, double ict, double tct, int icValue, int tcValue)
{
    int indexMax, indexMin;
    indexMin = 0;
    indexMax = ICTC_RANGE - 1;
    double interpolatedTemperature, icMinTcxValue, icMaxTcxValue;
    if (icValue == indexMin || icValue == indexMax)
    {
        interpolatedTemperature = svt * svInterpolationFactor
            + tct * tcInterpolationFactor + ict * icInterpolationFactor;
    }
    else
    {
        icMinTcxValue = svt * svMinInterpolationFactor
            + tct * tcMinInterpolationFactor + ict * icMinInterpolationFactor;
        icMaxTcxValue = svt * svMaxInterpolationFactor
            + tct * tcMaxInterpolationFactor + ict * icMaxInterpolationFactor;
        interpolatedTemperature = icMinTcxValue
            + (icMaxTcxValue - icMinTcxValue) * icvInterpolationFactor;
    }
    return interpolatedTemperature;
}

/*
 * Runs the interpolation procedure of LCScalingOutput. Interpolates the data
 * and outputs the interpolations to a subdirectory of the output folder. This
 * method assumes that the following pairing exists for the edge cases:
 * (IC0TC0, BlockGroup 1), (IC0TC100, BlockGroup 2), (IC100TC0, BlockGroup 3)
 *
 * @param inputs the configuration options for the simulation
 * @param blockGroups the block groups (is this even needed?)
 */
void ICTCInterpolator::interpolateData(Inputs* inputs)
{
    int indexMin, indexMax;
    indexMin = 0;
    indexMax = 100;
    map<std::string, map<std::string, double>> edgeCaseData;
    std::string outputDirectory = inputs->SimulationStringParams["OutputDirectory"];
    readEdgeCaseData(inputs, &edgeCaseData);
    // Iterate through all ICXTCX values and interpolate the time series
    for (int tcValue = 0; tcValue < ICTC_RANGE; tcValue++)
    {
        // set values for interpolation
        icMinInterpolationFactor = computeICInterpolationFactor(indexMin, tcValue);
        icMaxInterpolationFactor = computeICInterpolationFactor(indexMax, tcValue);
        tcMinInterpolationFactor = computeTCInterpolationFactor(indexMin, tcValue);
        tcMaxInterpolationFactor = computeTCInterpolationFactor(indexMax, tcValue);
        svMinInterpolationFactor = computeSVInterpolationFactor(indexMin, tcValue);
        svMaxInterpolationFactor = computeSVInterpolationFactor(indexMax, tcValue);
        for (int icValue = 0; icValue < ICTC_RANGE; icValue++)
        {
            // Create new output file for ICXTCX case
            std::string filename = "IC" + std::to_string(icValue) + "TC" + std::to_string(tcValue) + ".txt";
            ofstream outfile(outputDirectory + "\\" + filename);
            outfile << "YYYYMMDDHH    Ta    Td" << std::endl;

            icvInterpolationFactor = computeICVInterpolationFactor(icValue);
            icInterpolationFactor = computeICInterpolationFactor(icValue, tcValue);
            tcInterpolationFactor = computeTCInterpolationFactor(icValue, tcValue);
            svInterpolationFactor = computeSVInterpolationFactor(icValue, tcValue);

            for (std::pair<std::string, map<std::string, double>> element : edgeCaseData)
            {
                std::string datetime = element.first;
                double ic0tc0AirTemp, ic0tc0DewTemp, ic0tc100AirTemp, ic0tc100DewTemp,
                    ic100tc0AirTemp, ic100tc0DewTemp, interpolatedAirTemp, interpolatedDewTemp;
                ic0tc0AirTemp = element.second[ICTCInterpolator::IC0TC0_AIR_TEMP];
                ic0tc0DewTemp = element.second[ICTCInterpolator::IC0TC0_DEW_TEMP];
                ic0tc100AirTemp = element.second[ICTCInterpolator::IC0TC100_AIR_TEMP];
                ic0tc100DewTemp = element.second[ICTCInterpolator::IC0TC100_DEW_TEMP];
                ic100tc0AirTemp = element.second[ICTCInterpolator::IC100TC0_AIR_TEMP];
                ic100tc0DewTemp = element.second[ICTCInterpolator::IC100TC0_DEW_TEMP];
                interpolatedAirTemp = interpolateHourlyTemperature(ic0tc0AirTemp, ic100tc0AirTemp, ic0tc100AirTemp, icValue, tcValue);
                interpolatedDewTemp = interpolateHourlyTemperature(ic0tc0DewTemp, ic100tc0DewTemp, ic0tc100DewTemp, icValue, tcValue);
                outfile << datetime << "   " << interpolatedAirTemp << "  " << interpolatedDewTemp << std::endl;
            }
        }
    }
}

/*
 * Helper method that loads the edge case data for the simulation
 */
void readEdgeCaseData(Inputs* inputs, map<std::string, map<std::string, double>> *edgeCaseData)
{
    std::string outputDirectory = inputs->SimulationStringParams["OutputDirectory"];
    std::string ic0tc0Filename   = "BlockGroup1.txt";
    std::string ic0tc100Filename = "BlockGroup2.txt";
    std::string ic100tc0Filename = "BlockGroup3.txt";
    ifstream ic0tc0Data(outputDirectory + "\\" + ic0tc0Filename);
    ifstream ic0tc100Data(outputDirectory + "\\" + ic0tc100Filename);
    ifstream ic100tc0Data(outputDirectory + "\\" + ic100tc0Filename);
    if (ic0tc0Data.is_open() && ic0tc100Data.is_open() && ic100tc0Data.is_open())
    {
        std::string ic0tc0Line, ic0tc100Line, ic100tc0Line;
        // Ignore the first line of each data file
        getline(ic0tc0Data, ic0tc0Line);
        getline(ic0tc100Data, ic0tc100Line);
        getline(ic100tc0Data, ic100tc0Line);
        
        // read each line of hourly data until one of the files reaches its end
        while (!(ic0tc0Data.eof() || ic0tc100Data.eof() || ic100tc0Data.eof()))
        {
            map<std::string, double> temperatures;
            stringstream ss;
            std::string datetime;
            double ic0tc0AirTemp,   ic0tc0DewTemp,
                   ic0tc100AirTemp, ic0tc100DewTemp,
                   ic100tc0AirTemp, ic100tc0DewTemp;
            // read each line
            std::getline(ic0tc0Data, ic0tc0Line);
            std::getline(ic0tc100Data, ic0tc100Line);
            std::getline(ic100tc0Data, ic100tc0Line);
            // use a string stream to extract the datetime, air temperature, and dewpoint temperature for each line
            ss = stringstream(ic0tc0Line);
            ss >> datetime;
            ss >> ic0tc0AirTemp;
            ss >> ic0tc0DewTemp;
            ss = stringstream(ic0tc100Line);
            ss >> datetime;
            ss >> ic0tc100AirTemp;
            ss >> ic0tc100DewTemp;
            ss = stringstream(ic100tc0Line);
            ss >> datetime;
            ss >> ic100tc0AirTemp;
            ss >> ic100tc0DewTemp;
            if (datetime != "")
            {
                temperatures[ICTCInterpolator::IC0TC0_AIR_TEMP] = ic0tc0AirTemp;
                temperatures[ICTCInterpolator::IC0TC0_DEW_TEMP] = ic0tc0DewTemp;
                temperatures[ICTCInterpolator::IC0TC100_AIR_TEMP] = ic0tc100AirTemp;
                temperatures[ICTCInterpolator::IC0TC100_DEW_TEMP] = ic0tc100DewTemp;
                temperatures[ICTCInterpolator::IC100TC0_AIR_TEMP] = ic100tc0AirTemp;
                temperatures[ICTCInterpolator::IC100TC0_DEW_TEMP] = ic100tc0DewTemp;
                (*edgeCaseData)[datetime] = temperatures;
            }
        }
    }
}